// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.ui.popup

import com.intellij.openapi.actionSystem.ActionGroup
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.actionSystem.impl.segmentedActionBar.PillBorder
import com.intellij.openapi.util.Condition
import com.intellij.openapi.util.NlsContexts
import com.intellij.ui.popup.list.PopupListElementRenderer
import com.intellij.util.ui.JBUI
import com.intellij.util.ui.UIUtil
import net.miginfocom.swing.MigLayout
import java.awt.BorderLayout
import java.awt.Font
import javax.swing.*

open class StateActionGroupPopup(@NlsContexts.PopupTitle title: String?,
                                 actionGroup: ActionGroup,
                                 dataContext: DataContext,
                                 showNumbers: Boolean,
                                 useAlphaAsNumbers: Boolean,
                                 showDisabledActions: Boolean,
                                 honorActionMnemonics: Boolean,
                                 disposeCallback: Runnable?,
                                 maxRowCount: Int,
                                 preselectActionCondition: Condition<AnAction>?,
                                 actionPlace: String?,
                                 autoSelection: Boolean,
                                 val getState: (AnAction) -> String?) :
  PopupFactoryImpl.ActionGroupPopup(title, actionGroup, dataContext, showNumbers,
                                    useAlphaAsNumbers,
                                    showDisabledActions, honorActionMnemonics,
                                    disposeCallback, maxRowCount,
                                    preselectActionCondition,
                                    actionPlace, autoSelection) {

  override fun getListElementRenderer(): ListCellRenderer<*> {
    return object : PopupListElementRenderer<Any?>(this) {
      private var stateLabel: JLabel? = null
      private var stateButton: JComponent? = null

      override fun createItemComponent(): JComponent {
        val panel = JPanel(BorderLayout())
        val rightPane = JPanel(BorderLayout())
        createLabel()
        panel.add(myTextLabel, BorderLayout.CENTER)
        myTextLabel.border = JBUI.Borders.emptyTop(1)

        val bt = createButton()
        rightPane.add(bt, BorderLayout.CENTER)
        stateButton = bt

        val myShortcutLabel = JLabel()
        myShortcutLabel.border = JBUI.Borders.emptyRight(3)
        myShortcutLabel.foreground = UIManager.getColor("MenuItem.acceleratorForeground")
        rightPane.add(myShortcutLabel, BorderLayout.EAST)
        panel.add(rightPane, BorderLayout.EAST)
        rightPane.border = JBUI.Borders.emptyLeft(5)
        return layoutComponent(panel)
      }

      private fun createButton(): JComponent {
        val pane = JPanel(MigLayout("ins 0, gap 0")).apply {
          isOpaque = false
        }

        val lb = object : JLabel() {
          override fun getFont(): Font {
            return JBUI.Fonts.miniFont()
          }
        }.apply {
          foreground = JBUI.CurrentTheme.Advertiser.foreground()
        }

        stateLabel = lb
        pane.add(lb, "gapbefore ${JBUI.scale(3)}, gapafter ${JBUI.scale(3)}, ay center")
        pane.border = PillBorder(
          JBUI.CurrentTheme.Advertiser.background(), 1)

        return JPanel(MigLayout("ins 0, gap 0, ay center", "", "push[]push")).apply {
          isOpaque = true
          add(pane)
        }
      }

      override fun customizeComponent(list: JList<out Any?>?, value: Any?, isSelected: Boolean) {

        super.customizeComponent(list, value, isSelected)
        stateButton?.let { button ->
          stateLabel?.let { lb ->
            button.isVisible = value?.let { vl ->
              if (vl is PopupFactoryImpl.ActionItem) {
                val action = vl.action
                getState(action)?.let {
                  lb.foreground = UIUtil.getLabelForeground()
                  lb.text = it
                  it.isNotEmpty()
                } ?: false
              }
              else false

            } ?: false
          }
        }
      }


    }
  }

}